from datetime import datetime
from unittest import TestCase

from splight_models import *
from splight_models.base import SplightBaseModel


class AbstractBaseHash(object):
    class_model = SplightBaseModel
    args = {}
    rand_str_arg = ""

    def test_eq_hash(self):
        obj1 = self.class_model(**self.args)
        obj2 = self.class_model(**self.args)
        self.assertEqual(obj1.__hash__(), obj2.__hash__())

    def test_neq_hash(self):
        obj1_args = self.args.copy()
        obj2_args = self.args.copy()
        obj1_args[self.rand_str_arg] = "randstr1"
        obj2_args[self.rand_str_arg] = "randstr2"
        obj1 = self.class_model(**obj1_args)
        obj2 = self.class_model(**obj2_args)
        self.assertNotEqual(obj1.__hash__(), obj2.__hash__())


class TestAssetHash(AbstractBaseHash, TestCase):
    class_model = Asset
    args = {"name": ""}
    rand_str_arg = "name"


class TestAttributeHash(AbstractBaseHash, TestCase):
    class_model = Attribute
    args = {"name": ""}
    rand_str_arg = "name"


class TestDeploymentHash(AbstractBaseHash, TestCase):
    class_model = Deployment
    args = {"version": "ver", "type": "Algorithm"}
    rand_str_arg = "version"


class TestGeopointHash(AbstractBaseHash, TestCase):
    class_model = Geopoint
    args = {"id": "id", "latitude": 0.1, "longitude": 0.2}
    rand_str_arg = "id"


class TestMappingHash(AbstractBaseHash, TestCase):
    class_model = Mapping
    args = {
        "name": "MyMapping",
        "asset_id": "1",
        "attribute_id": "1",
        "connector_id": "val",
        "path": "val",
    }
    rand_str_arg = "asset_id"


class TestMessageHash(AbstractBaseHash, TestCase):
    class_model = Message
    args = {"action": "1", "variables": []}
    rand_str_arg = "action"


class TestNamespaceHash(AbstractBaseHash, TestCase):
    class_model = Namespace
    args = {"id": "1"}
    rand_str_arg = "id"


class TestNetTargetHash(AbstractBaseHash, TestCase):
    class_model = NetTarget
    args = {"ip": "1", "port": "1"}
    rand_str_arg = "ip"


class TestNotificationHash(AbstractBaseHash, TestCase):
    class_model = Notification
    args = {"title": "1", "message": "1", "timestamp": datetime.now()}
    rand_str_arg = "title"


class TestRuleVariableHash(AbstractBaseHash, TestCase):
    class_model = RuleVariable
    args = {"id": "1"}
    rand_str_arg = "id"


class TestRule(AbstractBaseHash, TestCase):
    class_model = Rule
    args = {
        "query": {
            "source": "Algorithm",
            "component_id": "01d08df6-e9f6-489c-ad62-9c8e6b714412",
            "output_format": "Value",
            "target": "value",
            "filters": {},
            "timezone_offset": 0.0
        },
        "type": "float",
        "value": 3,
        "message": "Random algo outputs > 3",
        "name": "Random checker test",
        "description": "Random algo outputs > 3",
        "severity": "info",
        "operator": "gt",
        "period": 10.0
    }
    rand_str_arg = "message"


class TestComponentHash(AbstractBaseHash, TestCase):
    class_model = Component
    args = {"name": "1", "version": "1", "type": "Algorithm"}
    rand_str_arg = "name"


class TestAlgorithmHash(AbstractBaseHash, TestCase):
    class_model = Algorithm
    args = {"name": "1", "version": "1", "type": "Algorithm"}
    rand_str_arg = "name"


class TestNetworkHash(AbstractBaseHash, TestCase):
    class_model = Network
    args = {"name": "1", "version": "1", "type": "Network"}
    rand_str_arg = "name"


class TestConnectorHash(AbstractBaseHash, TestCase):
    class_model = Connector
    args = {"name": "1", "version": "1", "type": "Connector"}
    rand_str_arg = "name"


class TestSystemHash(AbstractBaseHash, TestCase):
    class_model = System
    args = {"name": "1", "version": "1", "type": "System"}
    rand_str_arg = "name"


class TestStorageFileHash(AbstractBaseHash, TestCase):
    class_model = StorageFile
    args = {"file": "1"}
    rand_str_arg = "file"


class TestTagHash(AbstractBaseHash, TestCase):
    class_model = Tag
    args = {"name": "1"}
    rand_str_arg = "name"


class TestVariableHash(AbstractBaseHash, TestCase):
    class_model = Variable
    args = {"path": "1", "args": {}, "timestamp": datetime.now()}
    rand_str_arg = "path"
